Package: cn.mops
Authors@R: c(person(given = "Guenter", family = "Klambauer", role = "aut"), person(given = "Gundula", family = "Povysil", role = "cre", email = "povysil@bioinf.jku.at"))
License: LGPL (>= 2.0)
Type: Package
Title: cn.mops - Mixture of Poissons for CNV detection in NGS data
Description: cn.mops (Copy Number estimation by a Mixture Of PoissonS)
        is a data processing pipeline for copy number variations and
        aberrations (CNVs and CNAs) from next generation sequencing
        (NGS) data. The package supplies functions to convert BAM files
        into read count matrices or genomic ranges objects, which are
        the input objects for cn.mops. cn.mops models the depths of
        coverage across samples at each genomic position. Therefore, it
        does not suffer from read count biases along chromosomes. Using
        a Bayesian approach, cn.mops decomposes read variations across
        samples into integer copy numbers and noise by its mixture
        components and Poisson distributions, respectively. cn.mops
        guarantees a low FDR because wrong detections are indicated by
        high noise and filtered out. cn.mops is very fast and written
        in C++.
Version: 1.59.0
Date: 2025-07-22
URL: http://www.bioinf.jku.at/software/cnmops/cnmops.html
Depends: R (>= 3.5.0), methods, utils, stats, graphics, parallel,
        GenomicRanges
Imports: BiocGenerics, Biobase, IRanges, Rsamtools, Seqinfo, S4Vectors
Suggests: DNAcopy
LazyLoad: yes
biocViews: Sequencing, CopyNumberVariation, Homo_sapiens, CellBiology,
        HapMap, Genetics
RoxygenNote: 7.1.1
Config/pak/sysreqs: libbz2-dev liblzma-dev xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:36:04 UTC
RemoteUrl: https://github.com/bioc/cn.mops
RemoteRef: HEAD
RemoteSha: aae0aae82b3c5c756bbd0bac00417684074aa619
NeedsCompilation: yes
Packaged: 2026-05-30 08:51:29 UTC; root
Author: Guenter Klambauer [aut],
  Gundula Povysil [cre]
Maintainer: Gundula Povysil <povysil@bioinf.jku.at>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-30 08:55:37 UTC; unix
