# Generated by roxygen2: do not edit by hand

export(cn.mops)
export(exomecn.mops)
export(getReadCountsFromBAM)
export(getSegmentReadCountsFromBAM)
export(haplocn.mops)
export(normalizeChromosomes)
export(normalizeGenome)
export(referencecn.mops)
export(segment)
export(singlecn.mops)
exportClasses(CNVDetectionResult)
exportMethods(calcFractionalCopyNumbers)
exportMethods(calcIntegerCopyNumbers)
exportMethods(cnvr)
exportMethods(cnvs)
exportMethods(gr)
exportMethods(individualCall)
exportMethods(iniCall)
exportMethods(integerCopyNumber)
exportMethods(localAssessments)
exportMethods(makeRobustCNVR)
exportMethods(normalizedData)
exportMethods(params)
exportMethods(plot)
exportMethods(posteriorProbs)
exportMethods(sampleNames)
exportMethods(segmentation)
exportMethods(segplot)
exportMethods(show)
import(methods)
import(stats)
import(utils)
importFrom(Biobase,isUnique)
importFrom(Biobase,rowMedians)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,start)
importFrom(BiocGenerics,strand)
importFrom(GenomicRanges,"mcols<-")
importFrom(GenomicRanges,"strand<-")
importFrom(GenomicRanges,"values<-")
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,reduce)
importFrom(GenomicRanges,values)
importFrom(IRanges,IRanges)
importFrom(IRanges,as.data.frame)
importFrom(IRanges,as.list)
importFrom(IRanges,as.matrix)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,ranges)
importFrom(IRanges,reduce)
importFrom(IRanges,setdiff)
importFrom(IRanges,unique)
importFrom(IRanges,width)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,scanBam)
importFrom(Rsamtools,scanBamFlag)
importFrom(Rsamtools,scanBamHeader)
importFrom(S4Vectors,subjectHits)
importFrom(S4Vectors,values)
importFrom(S4Vectors,mcols)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(Seqinfo,sortSeqlevels)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.interactive)
importFrom(grDevices,dev.new)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,title)
importFrom(methods,show)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,makeCluster)
importFrom(parallel,parApply)
importFrom(parallel,stopCluster)
useDynLib(cn.mops)
