setGeneric("gr", function(object) 
			standardGeneric("gr"))
setGeneric("normalizedData", function(object) 
			standardGeneric("normalizedData"))
setGeneric("localAssessments", function(object) 
			standardGeneric("localAssessments"))
setGeneric("individualCall", function(object) 
			standardGeneric("individualCall"))
setGeneric("iniCall", function(object) 
			standardGeneric("iniCall"))
setGeneric("posteriorProbs", function(object) 
			standardGeneric("posteriorProbs"))
setGeneric("cnvs", function(object) 
			standardGeneric("cnvs"))
setGeneric("cnvr", function(object) 
			standardGeneric("cnvr"))
setGeneric("sampleNames", function(object) 
			standardGeneric("sampleNames"))
setGeneric("segmentation", function(object) 
			standardGeneric("segmentation"))
setGeneric("integerCopyNumber", function(object) 
			standardGeneric("integerCopyNumber"))
setGeneric("params", function(object) 
			standardGeneric("params"))
setGeneric("calcIntegerCopyNumbers", function(object) 
			standardGeneric("calcIntegerCopyNumbers"))
setGeneric("makeRobustCNVR", function(object, robust=0.5, minWidth=4, ...) 
			standardGeneric("makeRobustCNVR"))
setGeneric("calcFractionalCopyNumbers", function(object,segStat="mean") 
			standardGeneric("calcFractionalCopyNumbers"))
