% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methodsAccess.R
\docType{methods}
\name{cnvr,CNVDetectionResult-method}
\alias{cnvr,CNVDetectionResult-method}
\title{This generic function returns CNV regions of
 a CNV detection method stored in an instance of 
\code{\link{CNVDetectionResult-class}}.}
\usage{
\S4method{cnvr}{CNVDetectionResult}(object)
}
\arguments{
\item{object}{An instance of "CNVDetectionResult"}
}
\value{
\code{cnvr} returns a  eturns a "GRanges" object containing
the CNV regions.
}
\description{
This generic function returns CNV regions of
 a CNV detection method stored in an instance of 
\code{\link{CNVDetectionResult-class}}.
}
\examples{
data(cn.mops)
r <- cn.mops(X[1:100,1:5])
cnvr(r)
}
\author{
Guenter Klambauer \email{klambauer@bioinf.jku.at}
}
